from django.contrib.auth.models import User
#from django.contrib.gis.db.models import PointField
from django.db import models
from django.utils.translation import ugettext as _

from tagging.fields import TagField


class Plugin(models.Model):
    """Simple Django plug-in apps application.

    Using this plug-in app should be possible to enable or disable plug-ins,
    also should be possible to specify how to find plug-ins in settings.py.

    Maybe we should reuse one of these:
    - http://pypi.python.org/pypi/django-app-plugins/
    - https://github.com/alex/django-plugins
    - http://djangosnippets.org/snippets/542/

    If none of above plug-in frameworks fits our need, then we should consider
    using some thing like setuptools entry-points:
    - http://packages.python.org/distribute/pkg_resources.html#entry-points

    """

    plugin = models.CharField(max_length=128)


class UserProfile(models.Model):
    """User profile.

    This model must work with plug-ins, like Supporter, etc.

    """

    user = models.OneToOneField(User)


class Supporter(models.Model):
    """User user, than can support his neighbors."""

    user = models.OneToOneField(UserProfile)


class Entry(models.Model):
    """General content entry model.

    An entry must be provided with an entry plug-in, like Article, Link,
    Discussion, etc. This model only controls what entries user have
    contributed.

    Each entry plugin stores raw data, but in Entry model are savend final
    generated HTML content, that can be displayed to users.

    ``body``
        Should be stored in some markup like reST or Markdown.

    """

    plugin = models.ForeignKey(Plugin)

    author = models.ForeignKey(UserProfile)

    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    publish = models.DateTimeField(auto_now=True)

    title = models.CharField(max_length=255)
    teaser = models.TextField(blank=True)
    body = models.TextField(blank=True)

    tags = TagField()


class Attachment(models.Model):
    """Attachments to entries.

    These attacments should be implemented as plugins. For example can be
    image, file attachment plugins.

    """

    entry = models.OneToOneField(Entry)


class Comment(models.Model):
    """General purpose comment."""

    entry = models.ForeignKey(Entry)
    author = models.ForeignKey(UserProfile)

    body = models.TextField()

    vote_up = models.IntegerField()
    vote_down = models.IntegerField()


class Article(models.Model):
    """Article, can be used for any type of writing, like news, etc."""

    entry = models.OneToOneField(Entry)


class Link(models.Model):
    """Link to external article."""

    entry = models.OneToOneField(Entry)
    link = models.URLField()


class Screenshot(models.Model):
    """Free form discussion forum."""

    entry = models.OneToOneField(Entry)
